########################################################################################################################
# IMPORTS

import configparser
import logging
import random
import re
import shlex
import subprocess
import time

import pendulum

########################################################################################################################
# FUNCTIONS

logger = logging.getLogger(__name__)


def get_config(config_path):
    cfg = configparser.RawConfigParser()
    cfg.read(config_path)
    return cfg


def set_logger(level):
    log = logging.getLogger()
    log.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(level.upper())
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    log.addHandler(ch)


def ban_sleep(max_time, min_time=0):
    sleep_time = int(random.uniform(min_time, max_time))
    logger.info(f"sleeping for {sleep_time} seconds...")
    time.sleep(sleep_time)


def run_bash_command(command):
    p = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    text_lines = []
    for line_b in iter(p.stdout.readline, ""):
        line_str = line_b.decode().strip()

        if not line_str:
            break

        logger.info(line_str)
        text_lines.append(line_str)

    return "\n".join(text_lines)


def text_to_int(text):
    max_int32 = 2147483647
    try:
        num = int(re.sub(r"[^\d-]", "", text))

        if -max_int32 < num < max_int32:  # max integer value (in order to avoid problems on database insertion)
            return num

    except ValueError:
        logger.warning(f"unable to parse {text} as integer")


def sleep_out_interval(from_h, to_h, tz="Europe/Madrid", seconds=1800):
    while pendulum.now(tz=tz).hour >= to_h or pendulum.now(tz=tz).hour < from_h:
        logger.warning("time to sleep and not scrape anything...")
        ban_sleep(seconds, seconds)


def sleep_in_interval(from_h, to_h, tz="Europe/Madrid", seconds=1800):
    while from_h <= pendulum.now(tz=tz).hour < to_h:
        logger.warning("time to sleep and not scrape anything...")
        ban_sleep(seconds, seconds)


def parse_field(dict_struct, field_path, format_method=None):
    if not isinstance(field_path, list):
        raise ValueError("Argument field_path must be of type list")

    field_value = dict_struct
    for field in field_path:
        if isinstance(field_value, dict):
            field_value = field_value.get(field)
        elif isinstance(field_value, list):
            field_value = field_value[field] if len(field_value) > field else None
        if field_value is None:
            return None
    return format_method(field_value) if format_method else field_value



########################################################################################################################
# IMPORTS

import io
import logging

import boto3

########################################################################################################################
# CLASSES

logger = logging.getLogger(__name__)


class AWSInterface:

    def __init__(self, config):
        if 'aws' in config:
            self.config = config['aws']

            self.profile = self.config['profile']
            self.bucket = self.config['bucket']

            self.session = boto3.Session(profile_name=self.profile)

            self.s3 = self.session.resource('s3')
            self.s3_client = self.s3.meta.client

        else:
            logger.warning('no aws section in config')

    def get_file(self, s3_path):
        try:
            return self.s3.Object(self.bucket, s3_path).get()
        except self.s3_client.exceptions.NoSuchKey:
            logger.info(f'{s3_path} does not exist')

    def read_file_as_bytes(self, s3_path):
        return io.BytesIO(self.get_file(s3_path)['Body'].read())

    def upload_file(self, local_path, s3_path):
        self.s3.Bucket(self.bucket).upload_file(local_path, s3_path)
        



########################################################################################################################
# IMPORTS

import logging

import undetected_chromedriver as uc
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from .main import run_bash_command

########################################################################################################################
# FUNCTIONS

logger = logging.getLogger(__name__)


def get_chromedriver_version():
    return int(
        run_bash_command("/usr/bin/google-chrome --version").split(" ")[2].split(".")[0]
    )


def get_driver(chrome_options=None, **kwargs):
    options = uc.ChromeOptions()

    if isinstance(chrome_options, list):
        for option in chrome_options:
            if isinstance(option, dict):
                for name, value in option["experimental"].items():
                    options.add_experimental_option(name, value)
            else:
                options.add_argument(option)

    return uc.Chrome(options=options, version_main=get_chromedriver_version(), **kwargs)


def wait(driver, css_selector, timeout=30):
    logger.info(f"waiting for {css_selector}...")
    return WebDriverWait(driver, timeout).until(
        EC.visibility_of_element_located(("css selector", css_selector))
    )


def wait_and_click(driver, css_selector, timeout=30):
    logger.info(f"clicking on {css_selector}...")
    WebDriverWait(driver, timeout).until(
        EC.element_to_be_clickable(("css selector", css_selector))
    ).click()


def wait_and_fill(driver, css_selector, text_to_fill, timeout=30):
    logger.info(f"sending text to {css_selector}...")
    WebDriverWait(driver, timeout).until(
        EC.presence_of_element_located(("css selector", css_selector))
    ).send_keys(text_to_fill)


def scroll(driver, css_selector):
    element = wait(driver, css_selector)
    logger.info(f"scrolling to {css_selector}")
    driver.execute_script("arguments[0].scrollIntoView();", element)


def scroll_and_click(driver, css_selector):
    element = wait(driver, css_selector)
    logger.info(f"scrolling to {css_selector}")
    driver.execute_script("arguments[0].scrollIntoView();", element)

    logger.info(f"clicking to {css_selector}")
    element.click()


def press_key(driver, key):
    actions = ActionChains(driver)
    actions.send_keys(key).perform()



########################################################################################################################
# IMPORTS

import logging
from ftplib import FTP, FTP_TLS
from pathlib import Path

########################################################################################################################
# CLASSES

logger = logging.getLogger(__name__)


class FTPInterface:
    def __init__(self, config):
        if "ftp" in config:
            self.config = config["ftp"]

            self.ftp = self.get_ftp()
        else:
            logger.warning("no ftp section in config")

    def get_ftp(self):
        if self.config["ftps"].lower() == "true":
            ftp_conn = FTP_TLS(self.config["server"])

        else:
            ftp_conn = FTP(self.config["server"])

        ftp_conn.login(self.config["username"], self.config["password"])

        return ftp_conn

    def upload_file(self, local_file, remote_folder, remote_file=None):
        if not remote_file:
            remote_file = Path(local_file).name

        self.ftp.cwd(remote_folder)

        with open(local_file, "rb") as f:
            self.ftp.storbinary(f"STOR {remote_file}", f)

    def download_file(self, local_file, remote_file):
        with open(local_file, "wb") as f:
            self.ftp.retrbinary(f"RETR {remote_file}", f.write)



"""
Project setup module.
"""

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

requirements = [
    "sqlalchemy",
    "requests[socks]",
    "retry",
    "stem",
    "pydrive2",
    "undetected_chromedriver",
    "pendulum",
    "boto3",
    "inflection",
    "unicodedata"
]

setuptools.setup(
    name="datamarket",
    version="0.5.18",
    author="DataMarket",
    author_email="techsupport@datamarket.es",
    description="Utilities that integrate advance scraping knowledge into just one library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Data-Market/datamarket",
    project_urls={
        "Website": "https://datamarket.es",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7",
    install_requires=requirements,
)



########################################################################################################################
# IMPORTS

import datetime
import json
import logging

import requests
from requests.exceptions import ConnectionError
from retry import retry

########################################################################################################################
# CLASSES

logger = logging.getLogger(__name__)


class TinybirdInterface:
    def __init__(self, config):
        if "tinybird" in config:
            self.config = config["tinybird"]

            self.post_url = "https://api.tinybird.co/v0/events"

            self.request_params = {
                "name": self.config["name"],
                "token": self.config["token"],
            }

        else:
            logger.warning("no tinybird section in config")

    @staticmethod
    def __converter(obj):
        if isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

    @staticmethod
    def __dict_lists_to_string(obj):
        return {
            key: json.dumps(value, ensure_ascii=False) if isinstance(value, (list, dict)) else value
            for key, value in obj.items()
        }

    def __prepare_json_row(self, obj_dict):
        return json.dumps(
            self.__dict_lists_to_string(obj_dict), default=self.__converter
        )

    @staticmethod
    def __handle_api_response(json_response):
        successful_rows = json_response["successful_rows"]
        quarantined_rows = json_response["quarantined_rows"]

        if quarantined_rows > 0:
            logger.error(
                f"wrong insertion of {quarantined_rows} records to Tinybird API..."
            )
        else:
            logger.info(
                f"successfully inserted {successful_rows} records to Tinybird API!"
            )

        return successful_rows, quarantined_rows

    @retry(ConnectionError, tries=3, delay=2)
    def __insert_data_to_endpoint(self, data):
        r = requests.post(self.post_url, params=self.request_params, data=data)
        return self.__handle_api_response(r.json())

    def insert_record_to_api(self, obj_dict):
        return self.__insert_data_to_endpoint(self.__prepare_json_row(obj_dict))

    def insert_batch_to_api(self, batch):
        return self.__insert_data_to_endpoint(
            "\n".join([self.__prepare_json_row(x) for x in batch])
        )

    def insert_pandas_df_to_api(self, df):
        return self.__insert_data_to_endpoint(df.to_json(orient="records", lines=True))
